@@ -69,6 +69,8 @@ from foundry.gui.PaletteViewer import PaletteViewer, SidePalette
 from foundry.gui.settings import SETTINGS, save_settings
 from foundry.gui.SettingsDialog import POWERUPS, SettingsDialog
 from foundry.gui.SpinnerPanel import SpinnerPanel
+from foundry.gui.Toolbar import create_toolbar
+from foundry.gui.util import setup_window
 from foundry.gui.WarningList import WarningList
 from foundry.smb3parse.constants import (
     TILE_LEVEL_1,
@@ -114,50 +116,7 @@ class MainWindow(QMainWindow):
         self.setWindowIcon(icon(main_window_flags["icon"]))
         self.setStyleSheet(SETTINGS[main_window_flags["style"]])
 
-        for menu_name, menu in main_window_flags["menu"].items():
-            qmenu = QMenu(menu["name"])
-            if menu.get("attribute", False):
-                setattr(self, f"{menu_name}_menu", qmenu)
-            if "action" in menu:
-                qmenu.triggered.connect(getattr(self, menu["action"]))
-            menu_type = menu["type"]
-
-            for index, option_group in enumerate(menu["options"]):
-                if index != 0:
-                    qmenu.addSeparator()
-                if menu_type == "actions":
-                    for base_name, option in option_group.items():
-                        name: str = option["name"]
-                        action = qmenu.addAction(name)
-                        if option.get("attribute", False):
-                            setattr(self, f"{base_name}_action", action)
-                        if "action" in option:
-                            method = getattr(self, option["action"])
-                        elif "link" in option:
-
-                            def call_link(url: str):
-                                def call_link():
-                                    return open_url(url)
-
-                                return call_link
-
-                            method = call_link(option["link"])
-                        else:
-                            raise NotImplementedError
-                        if option.get("wrapped", False):
-                            action.triggered.connect(lambda *_: method())
-                        else:
-                            action.triggered.connect(method)
-
-                elif menu_type == "settings":
-                    for option in option_group.values():
-                        name: str = option["display_name"]
-                        action = qmenu.addAction(name)
-                        action.setProperty(ID_PROP, option["id"])
-                        action.setCheckable(True)
-                        action.setChecked(SETTINGS[option["name"]])
-
-            self.menuBar().addMenu(qmenu)
+        setup_window(self, main_window_flags)
 
         self.block_viewer = None
         self.object_viewer = None
@@ -229,22 +188,12 @@ class MainWindow(QMainWindow):
 
         splitter.setChildrenCollapsible(False)
 
-        level_toolbar = QToolBar("Level Info Toolbar", self)
-        level_toolbar.setContextMenuPolicy(Qt.PreventContextMenu)
-        level_toolbar.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
-        level_toolbar.setOrientation(Qt.Horizontal)
-        level_toolbar.setFloatable(False)
-
-        level_toolbar.addWidget(self.spinner_panel)
-        level_toolbar.addWidget(self.object_dropdown)
-        level_toolbar.addWidget(size_and_palette)
-        level_toolbar.addWidget(self.level_size_bar)
-        level_toolbar.addWidget(self.enemy_size_bar)
-        level_toolbar.addWidget(splitter)
-
-        level_toolbar.setAllowedAreas(Qt.LeftToolBarArea | Qt.RightToolBarArea)
-
-        self.addToolBar(Qt.RightToolBarArea, level_toolbar)
+        create_toolbar(self, "Generator Editor", [self.spinner_panel], Qt.RightToolBarArea)
+        create_toolbar(self, "Generator Dropdown", [self.object_dropdown], Qt.RightToolBarArea)
+        create_toolbar(self, "Palette", [size_and_palette], Qt.RightToolBarArea)
+        create_toolbar(self, "Level Size", [self.level_size_bar], Qt.RightToolBarArea)
+        create_toolbar(self, "Enemy Size", [self.enemy_size_bar], Qt.RightToolBarArea)
+        create_toolbar(self, "Splitter", [splitter], Qt.RightToolBarArea)
 
         self.object_toolbar = ObjectToolBar(self)
         self.object_toolbar.object_selected.connect(self._on_placeable_object_selected)
@@ -448,14 +397,16 @@ class MainWindow(QMainWindow):
 
         temp_rom = self._open_rom(path_to_temp_rom)
 
-        if not self._put_current_level_to_level_1_1(path_to_temp_rom):
-            return
-
-        if not self._set_default_powerup(path_to_temp_rom):
-            return
+        self._put_current_level_to_level_1_1(temp_rom)
+        self._set_default_powerup(temp_rom)
 
         save_all_palette_groups(temp_rom)
 
+        temp_rom.write(
+            Title_PrepForWorldMap - 0x8,
+            bytes([SETTINGS["default_starting_world"] - 1]),
+        )
+
         temp_rom.save_to(str(path_to_temp_rom))
 
         arguments = SETTINGS["instaplay_arguments"].replace("%f", str(path_to_temp_rom))
@@ -493,9 +444,7 @@ class MainWindow(QMainWindow):
 
         header_editor.exec_()
 
-    def _put_current_level_to_level_1_1(self, path_to_rom) -> bool:
-        rom = self._open_rom(path_to_rom)
-
+    def _put_current_level_to_level_1_1(self, rom) -> bool:
         # load world data
         world = SMB3World.from_world_number(rom, SETTINGS["default_starting_world"])
 
@@ -527,20 +476,11 @@ class MainWindow(QMainWindow):
 
         world.replace_level_at_position((layout_address, enemy_address - 1, object_set_number), position)
 
-        # save rom
-        rom.save_to(path_to_rom)
-
-        return True
-
-    def _set_default_powerup(self, path_to_rom) -> bool:
-        rom = self._open_rom(path_to_rom)
-
+    def _set_default_powerup(self, rom) -> bool:
         *_, powerup, hasPWing = POWERUPS[SETTINGS["default_powerup"]]
         hasStar = SETTINGS["default_power_has_star"]
-        worldStart = SETTINGS["default_starting_world"] - 1
         nop = 0xEA
 
-        rom.write(Title_PrepForWorldMap - 0x8, bytes([worldStart]))
         rom.write(Title_PrepForWorldMap - 0x4, bytes([nop, nop, nop]))
 
         rom.write(Title_PrepForWorldMap + 0x1, bytes([powerup]))
@@ -594,7 +534,6 @@ class MainWindow(QMainWindow):
         # set as Raccoon Mario instead of P-wing
         Map_Power_DispResetLocation = 0x3C5A2
         rom.write(Map_Power_DispResetLocation, bytes([nop, nop, nop]))
-        return True
 
     def on_screenshot(self, _) -> bool:
         if self.level_view is None:
